import json
import random

import cv2
import numpy as np
from torch.utils.data import Dataset
import torch

import utils.data_transforms


class ImageDataset(Dataset):

    def __init__(self, param, mode):
        if mode == 'train':
            self.img_transform = utils.data_transforms.Compose([
                # utils.data_transforms.CenterCrop((150, 150), (128, 128)),
                utils.data_transforms.RandomBackground(param["random_bg_color_range"]),
                utils.data_transforms.ColorJitter(param["random_brightness"], param["random_contrast"],
                                                  param["random_saturation"]),
                utils.data_transforms.RandomNoise(param["noise_std"]),
                utils.data_transforms.Normalize(mean=param["mean"], std=param["std"]),
                utils.data_transforms.RandomFlip(),
                utils.data_transforms.RandomPermuteRGB(),
                utils.data_transforms.ToTensor(),
            ])
            in_fn = param['dataset_root'] + 'train_val_test_split/{}.train.json'.format(
                param["category"].split('-')[0])
            self.voxels = np.load(param['dataset_root'] + 'sem_seg_h5/' + param["category"] + '/train_nor.npy').astype(
                np.long)
        elif mode == 'test':
            self.img_transform = utils.data_transforms.Compose([
                # utils.data_transforms.CenterCrop((150, 150), (128, 128)),
                utils.data_transforms.RandomBackground(param["random_bg_color_range"]),
                utils.data_transforms.Normalize(mean=param["mean"], std=param["std"]),
                utils.data_transforms.ToTensor(),
            ])
            in_fn = param['dataset_root'] + 'train_val_test_split/{}.test.json'.format(
                param["category"].split('-')[0])
            self.voxels = np.load(param['dataset_root'] + 'sem_seg_h5/' + param["category"] + '/test_nor.npy').astype(
                np.long)
        else:
            self.img_transform = utils.data_transforms.Compose([
                # utils.data_transforms.CenterCrop((150, 150), (128, 128)),
                utils.data_transforms.RandomBackground(param["random_bg_color_range"]),
                utils.data_transforms.Normalize(mean=param["mean"], std=param["std"]),
                utils.data_transforms.ToTensor(),
            ])
            in_fn = param['dataset_root'] + 'train_val_test_split/{}.val.json'.format(param["category"].split('-')[0])
            self.voxels = np.load(param['dataset_root'] + 'sem_seg_h5/' + param["category"] + '/val_nor.npy').astype(
                np.long)

        with open(in_fn, 'r') as fin:
            item_list = json.load(fin)
        self.data = []
        for each in item_list:
            self.data.append(each["anno_id"])
        self.num = len(self.data)
        self.render_path = param['dataset_root'] + 'objs/' + param["category"] + '/'
        self.views_num = param["views_num"]
        self.idx_list = [i for i in range(20)]
        self.class_id = np.unique(self.voxels)
        self.normal_class_id = {}
        self.step = 1. / (len(self.class_id) - 1)
        alpha = np.zeros(16)
        for i in range(len(self.class_id)):
            self.voxels[self.voxels == self.class_id[i]] = i
            alpha[i] = np.sum(self.voxels == i) / self.voxels.size
        self.alpha = torch.from_numpy(alpha)
        self.mode = mode
        print("loading finished")

    def __len__(self):
        return self.num

    def __getitem__(self, idx):
        anno_id = self.data[idx]
        if self.mode == 'train':
            img_num = random.randint(0, self.views_num - 1)
        else:
            img_num = 0

        img = cv2.imread('{}{}/{}.png'.format(self.render_path, anno_id, str(img_num)),
                         cv2.IMREAD_UNCHANGED).astype(np.float32) / 255.
        img_list = [img]
        timg = self.img_transform(img_list).squeeze(0)

        return timg, self.voxels[idx]
