# blender script

import collections
import json
import math
import os
import random
import gc
import mathutils
import bpy

stype = 'Display-1'
in_fn = '/home/ste/DiskG/DATASET/PartNet/partnet_dataset/stats/train_val_test_split/{}.train.json'.format(
    stype.split('-')[0])
out_fn = '/home/ste/DiskG/DATASET/PartNet/objs/{}/'.format(stype)
out = ''

if not os.path.exists(out_fn):
    os.mkdir(out_fn)

with open(in_fn, 'r') as fin:
    item_list = json.load(fin, object_pairs_hook=collections.OrderedDict)

scene = bpy.data.scenes["Scene"]
cam = bpy.data.objects["Camera"]
cam.rotation_mode = 'XYZ'
scene.render.resolution_x = 224
scene.render.resolution_y = 224
scene.render.image_settings.color_mode = 'RGBA'
scene.render.alpha_mode = 'TRANSPARENT'
bpy.data.scenes["Scene"].render.resolution_percentage = 100


def process_one_model(path, anno_id):
    for o in scene.objects:
        if o.type == 'MESH':
            o.select = True
        else:
            o.select = False
    bpy.ops.object.delete()
    for block in bpy.data.meshes:
        if block.users == 0:
            bpy.data.meshes.remove(block)

    for block in bpy.data.materials:
        if block.users == 0:
            bpy.data.materials.remove(block)

    for block in bpy.data.textures:
        if block.users == 0:
            bpy.data.textures.remove(block)

    for block in bpy.data.images:
        if block.users == 0:
            bpy.data.images.remove(block)
    for each in os.listdir(path):
        bpy.ops.import_scene.obj(filepath=path + each)

    global out
    out = out_fn + anno_id
    if not os.path.exists(out):
        os.mkdir(out)

    # uncomment here if want to save the model.obj
    # bpy.ops.export_scene.obj(filepath=out + '/model.obj', use_materials=False)


def set_camera(rx, rz, dis):
    rx = math.radians(rx)
    rz = math.radians(rz)
    rxy = dis * math.sin(rz)
    camz = dis * math.cos(rz)
    camx = rxy * math.cos(rx)
    camy = rxy * math.sin(rx)
    cam.location.x = camx
    cam.location.y = camy
    cam.location.z = camz
    Vector = (0 - camx, 0 - camy, 0 - camz)
    direction = mathutils.Vector(Vector)
    rot_quat = direction.to_track_quat('-Z', 'Y')
    myeulers = rot_quat.to_euler()
    cam.rotation_euler = myeulers
    del myeulers
    del direction
    del Vector, rx, rz, rxy, camx, camy, camz


for item_id in range(len(item_list) - 1, -1, -1):
    item = item_list[item_id]
    in_fn = '/home/ste/DiskG/DATASET/PartNet/data_v0/{}/objs/'.format(item['anno_id'])
    process_one_model(in_fn, item['anno_id'])
    idx = 0
    for rz in range(90, 0, -30):
        for rx in range(0, 360, 45):
            set_camera(rx, rz, 3)
            bpy.data.scenes["Scene"].render.filepath = out + '/' + str(idx) + '.png'
            bpy.ops.render.render(write_still=True)
            idx = idx + 1
    gc.collect()
    print('{} data left...'.format(item_id))
