# -*- coding: utf-8 -*-
#
# Developed by Haozhe Xie <cshzxie@gmail.com>

import torch


class Decoder(torch.nn.Module):
    def __init__(self):
        super(Decoder, self).__init__()

        # Layer Definition
        self.layer1 = torch.nn.Sequential(
            torch.nn.ConvTranspose3d(2048, 256, kernel_size=4, stride=2, padding=1),
            # torch.nn.ConvTranspose3d(2128, 256, kernel_size=4, stride=2, padding=1),
        )
        self.layer2 = torch.nn.Sequential(
            torch.nn.ConvTranspose3d(256, 128, kernel_size=4, stride=2, padding=1),
            torch.nn.BatchNorm3d(128),
        )
        self.layer3 = torch.nn.Sequential(
            torch.nn.ConvTranspose3d(128, 64, kernel_size=4, stride=2, padding=1),
            torch.nn.BatchNorm3d(64),
            torch.nn.ReLU()
        )
        self.layer4 = torch.nn.Sequential(
            torch.nn.ConvTranspose3d(64, 32, kernel_size=4, stride=2, padding=1),
            torch.nn.BatchNorm3d(32),
            torch.nn.ReLU()
        )
        self.layer5 = torch.nn.ConvTranspose3d(32, 10, kernel_size=1)

    def forward(self, image_features):
        gen_volume = image_features.view(-1, 2048, 2, 2, 2)
        # gen_volume = image_features.view(-1, 2128, 2, 2, 2)
        # print(gen_volume.size())   # torch.Size([batch_size, 2048, 2, 2, 2])
        gen_volume = self.layer1(gen_volume)
        # print(gen_volume.size())   # torch.Size([batch_size, 512, 4, 4, 4])
        gen_volume = self.layer2(gen_volume)
        gen_volume = self.layer3(gen_volume)
        gen_volume = self.layer4(gen_volume)
        # print(gen_volume.size())   # torch.Size([batch_size, 128, 8, 8, 8])
        # print(gen_volume.size())   # torch.Size([batch_size, 32, 16, 16, 16])
        # print(gen_volume.size())   # torch.Size([batch_size, 16, 32, 32, 32])
        gen_volume = self.layer5(gen_volume)
        # print(gen_volume.size())   # torch.Size([batch_size, 16, 32, 32, 32])
        # print(raw_feature.size())  # torch.Size([batch_size, 32, 32, 32, 32])

        return gen_volume
