import torch
from models.modules.lam import lam
import torchvision.models


class Encoder(torch.nn.Module):
    def __init__(self):
        super(Encoder, self).__init__()
        # Layer Definition
        vgg16_bn = torchvision.models.vgg16_bn(pretrained=True)
        self.vgg = torch.nn.Sequential(*list(vgg16_bn.features.children()))[:27]
        self.layer1 = torch.nn.Sequential(
            torch.nn.Conv2d(512, 512, kernel_size=3),
            torch.nn.BatchNorm2d(512),
            torch.nn.ELU(),
        )
        self.layer2 = torch.nn.Sequential(
            torch.nn.Conv2d(512, 512, kernel_size=3),
            torch.nn.BatchNorm2d(512),
            torch.nn.ELU(),
            torch.nn.MaxPool2d(kernel_size=3)
        )
        # self.att_fea2 = Pfm(4, 512, 10)
        self.layer3 = torch.nn.Sequential(
            torch.nn.Conv2d(512, 256, kernel_size=1),
            torch.nn.BatchNorm2d(256),
            torch.nn.ELU()
        )
        # self.att_fea3 = Pfm(4, 256, 10)
        # self.upsample = torch.nn.Upsample(scale_factor=2, mode='bilinear')
        # self.layer4 = torch.nn.Sequential(
        #     torch.nn.Conv2d(10, 10, kernel_size=1),
        #     torch.nn.BatchNorm2d(10),
        #     torch.nn.ELU()
        # )
        # Don't update params in VGG16
        for param in vgg16_bn.parameters():
            param.requires_grad = False

    def forward(self, rendering_images):
        # torch.Size([batch_size, img_c, img_h, img_w])

        features = self.vgg(rendering_images.view(-1, 3, 224, 224))
        # print(features.size())    # torch.Size([batch_size, 512, 28, 28])
        features = self.layer1(features)
        # print(features.size())    # torch.Size([batch_size, 512, 26, 26])
        features = self.layer2(features)
        # print(features.size())    # torch.Size([batch_size, 512, 24, 24])
        # att_fea_24 = self.att_fea2(features2)
        features = self.layer3(features)
        # print(features.size())    # torch.Size([batch_size, 256, 8, 8])
        # att_fea_8 = self.att_fea3(features3)
        # att_feas = torch.cat([att_fea_8, att_fea_24], 1)
        # att_feas = self.upsample(att_fea_24)
        # att_feas = self.layer4(att_feas)
        # print(image_features.size())  # torch.Size([batch_size, 256, 8, 8])
        # return torch.cat([features3, att_feas], 1)
        return features
        # print(image_features.size())  # torch.Size([batch_size, 276, 8, 8])
