import torch
import torchvision.models


# local awareness module
class lam(torch.nn.Module):
    def __init__(self, windows_size, in_chanel, windows_num):
        super(lam, self).__init__()
        # Layer Definition
        self.extract = torch.nn.Sequential(
            torch.nn.Conv2d(in_chanel, 32, 3, 1, 1),
            torch.nn.BatchNorm2d(32),
            torch.nn.ReLU(),
            torch.nn.Conv2d(32, 1, 3, 1, 1),
            torch.nn.ReLU(),
        )
        self.unfold = torch.nn.Unfold(windows_size)
        self.attention = torch.nn.Sequential(
            torch.nn.Conv2d(1, 32, 3, 1, 1),
            torch.nn.BatchNorm2d(32),
            torch.nn.ReLU(),
            torch.nn.Conv2d(32, 1, windows_size, 1, 0),
        )
        self.windows_size = windows_size
        self.windows_num = windows_num

    def forward(self, in_feature):
        features_all = self.extract(in_feature)
        att = self.attention(features_all)
        att = att.view(in_feature.shape[0], -1)
        topk_loc = torch.topk(att, self.windows_num)[1]
        fea_unfold = self.unfold(features_all).transpose(-1, -2)
        all_locs = [i for i in range(features_all.shape[0]) for j in range(self.windows_num)]
        features = fea_unfold[(all_locs, topk_loc.view(-1))]
        return features.reshape([in_feature.shape[0], self.windows_num, self.windows_size, self.windows_size])
