import inspect

import torch
import torch.nn as nn
import models
from utils.gpu_mem_track import MemTracker


class recmodel(nn.Module):

    def __init__(self, params):
        super(recmodel, self).__init__()
        self.encoder = models.modules.Encoder()
        self.decoder = models.modules.Decoder()
        self.encoder.apply(self._initialize_weights)
        self.decoder.apply(self._initialize_weights)
        # self.refiner.apply(self._initialize_weights)

    def _initialize_weights(self, m):
        if isinstance(m, nn.Conv2d) or isinstance(m, nn.ConvTranspose2d):
            nn.init.kaiming_normal_(m.weight)
            if m.bias is not None:
                nn.init.constant_(m.bias, 0)
        elif isinstance(m, nn.BatchNorm2d) or type(m) == torch.nn.BatchNorm3d:
            nn.init.constant_(m.weight, 1)
            nn.init.constant_(m.bias, 0)
        elif isinstance(m, nn.Linear):
            nn.init.normal_(m.weight, 0, 0.01)
            nn.init.constant_(m.bias, 0)

    def forward(self, image):
        image_features = self.encoder(image)
        generated_volumes = self.decoder(image_features)
        return generated_volumes
