from models.backbone.vgg16 import VGG
from utils.gpu_mem_track import *
from . import SingleStageModel
from models.modules.foal_loss import FocalLoss
import torch


class recmodel(SingleStageModel):

    def __init__(self, params, dist_model=False, alpha=None):
        super(recmodel, self).__init__(params, dist_model)

        model_params = params['module']
        self.model_params = model_params
        self.epsilon = 1e-6
        self.patch_size = model_params['patch_size']
        # self.ce = torch.nn.CrossEntropyLoss()
        self.ce = FocalLoss(2, 1 - alpha)
        self.stepsize = 1

    def _cal_loss(self, output, target):
        coarse_model = output
        label_vox_tar = target[0]

        encoder_loss = self.ce(coarse_model, label_vox_tar)

        return encoder_loss

    def eval(self):
        pass

    def step(self, output):
        encoder_loss = self._cal_loss(output, self.label)
        # encoder_loss, refiner_loss = self._cal_loss(output, self.label)

        if self.model.training:
            self.optim.zero_grad()
            encoder_loss.backward()
            torch.nn.utils.clip_grad_norm_(self.model.parameters(), 2)
            self.optim.step()

        torch.cuda.empty_cache()
        return {'encoder_loss': encoder_loss}

    @staticmethod
    def get_parameters(model, bias=False):
        import torch.nn as nn
        modules_skipped = (
            nn.ReLU,
            nn.MaxPool2d,
            nn.Dropout2d,
            nn.Dropout,
            nn.Sequential,
            VGG
        )
        for m in model.modules():
            if isinstance(m, nn.Conv2d):
                if bias:
                    yield m.bias
                else:
                    yield m.weight
            elif isinstance(m, nn.ConvTranspose2d):
                if bias:
                    assert m.bias is None
                else:
                    yield m.weight
            elif isinstance(m, nn.Linear):
                if bias:
                    yield m.bias
                else:
                    yield m.weight
            elif isinstance(m, nn.BatchNorm2d):
                if bias:
                    yield m.bias
                else:
                    yield m.weight
            elif isinstance(m, modules_skipped):
                continue
            else:
                raise ValueError('Unexpected module: %s' % str(m))
