import os
import torch
import torch.backends.cudnn as cudnn
import torch.distributed as dist

import models
import utils


class SingleStageModel(object):

    def __init__(self, params, dist_model=False):
        model_params = params['module']
        self.model = models.modules.__dict__[params['module']['arch']](model_params)
        utils.init_weights(self.model, init_type='xavier', init_gain=1)
        self.model.cuda()

        if params['optim'] == 'SGD':
            self.optim = torch.optim.SGD([{'params': filter(
                lambda x: x.requires_grad, self.model.encoder.parameters()), 'lr': params['lr']},
                {'params': self.model.decoder.parameters()},
            ],
                lr=params['lr'],
                momentum=0, weight_decay=0.0005)
        elif params['optim'] == 'Adam':
            self.optim = torch.optim.Adam(
                [{'params': filter(
                    lambda x: x.requires_grad, self.model.encoder.parameters()), 'lr': params['lr']},
                    {'params': self.model.decoder.parameters()},
                ],
                lr=params['lr'])
        else:
            raise Exception("No such optimizer: {}".format(params['optim']))

        if dist_model:
            self.model = utils.DistModule(self.model)
            self.world_size = dist.get_world_size()
        else:
            self.world_size = 1
        cudnn.benchmark = True

    def set_input(self, input, label=None):
        self.input = input
        if label is not None:
            self.label = label

    def eval(self):
        pass

    def step(self, output):
        pass

    def load_state(self, path, Iter=False, resume=False):
        if Iter:
            path = os.path.join(path, "ckpt_iter_{}.pth.tar".format(Iter))

        if resume:
            utils.load_state(path, self.model, self.optim)
        else:
            utils.load_state(path, self.model)

    def load_pretrain(self, load_path):
        utils.load_state(load_path, self.model)

    def save_state(self, path, Iter):
        path = os.path.join(path, "ckpt_iter_{}.pth.tar".format(Iter))

        torch.save({
            'step': Iter,
            'state_dict': self.model.state_dict(),
            'optimizer': self.optim.state_dict()}, path)

    def switch_to(self, phase):
        if phase == 'train':
            self.model.train()
        else:
            self.model.eval()
